<?php
namespace JET_APB;

/**
 * Form controls and notifications class
 */
class Form {

	/**
	 * Check if date field is already rendered
	 *
	 * @var boolean
	 */
	public $date_done = false;

	/**
	 * Check if provider field is already rendered
	 *
	 * @var boolean
	 */
	public $provider_done = false;

	/**
	 * Constructor for the class
	 */
	public function __construct() {

		new Form_Widget();

		add_filter(
			'jet-engine/forms/booking/field-types',
			array( $this, 'register_form_fields' )
		);

		add_action(
			'jet-engine/forms/booking/field-template/appointment_date',
			array( $this, 'date_field_template' ),
			10, 3
		);

		add_action(
			'jet-engine/forms/booking/field-template/appointment_provider',
			array( $this, 'provider_field_template' ),
			10, 3
		);

		add_filter(
			'jet-engine/forms/booking/notification-types',
			array( $this, 'register_notification' )
		);

		add_action(
			'jet-engine/forms/edit-field/before',
			array( $this, 'edit_fields' )
		);

		add_action(
			'jet-engine/forms/booking/notifications/fields-after',
			array( $this, 'notification_fields' )
		);

		add_filter(
			'jet-engine/forms/booking/notification/insert_appointment',
			array( $this, 'handle_notification' ),
			1, 2
		);

		add_filter(
			'jet-engine/forms/gateways/notifications-before',
			array( $this, 'before_form_gateway' ), 1, 2
		);

		add_action(
			'jet-engine/forms/gateways/on-payment-success',
			array( $this, 'on_gateway_success' ), 10, 3
		);

		add_action( 'elementor/preview/enqueue_scripts', array( $this, 'calendar_assets' ) );

	}

	/**
	 * Set bookng appointment notification before gateway
	 *
	 * @param  [type] $keep [description]
	 * @param  [type] $all  [description]
	 * @return [type]       [description]
	 */
	public function before_form_gateway( $keep, $all ) {

		foreach ( $all as $index => $notification ) {
			if ( 'insert_appointment' === $notification['type'] && ! in_array( $index, $keep ) ) {
				$keep[] = $index;
			}
		}

		return $keep;

	}

	/**
	 * Finalize booking on internal JetEngine form gateway success
	 *
	 * @return [type] [description]
	 */
	public function on_gateway_success( $form_id, $settings, $form_data ) {

		$appointment_id = $form_data['appointment_id'];

		if ( ! $appointment_id ) {
			return;
		}

		$appointment = Plugin::instance()->db->get_appointment_by( 'ID', $appointment_id );

		if ( ! $appointment ) {
			return;
		}

		Plugin::instance()->db->appointments->update(
			array( 'status' => 'completed', ),
			array( 'ID' => $appointment_id, )
		);

		Plugin::instance()->db->maybe_exclude_appointment_date( $appointment );

	}

	/**
	 * Handle apartment booking notification
	 *
	 * @return [type] [description]
	 */
	public function handle_notification( $args, $notifications ) {

		$service_field  = ! empty( $args['appointment_service_field'] ) ? $args['appointment_service_field'] : false;
		$provider_field = ! empty( $args['appointment_provider_field'] ) ? $args['appointment_provider_field'] : false;
		$date_field     = ! empty( $args['appointment_date_field'] ) ? $args['appointment_date_field'] : false;
		$email_field    = ! empty( $args['appointment_email_field'] ) ? $args['appointment_email_field'] : false;
		$data           = $notifications->data;

		if ( ! $service_field || ! $date_field || ! $email_field ) {
			$notifications->log[] = false;
			return false;
		}

		if ( '_manual_input' === $service_field ) {
			$service_id = ! empty( $args['appointment_service_id'] ) ? absint( $args['appointment_service_id'] ) : 0;
		} else {
			$service_id = ! empty( $data[ $service_field ] ) ? $data[ $service_field ] : false;
		}

		$date        = ! empty( $data[ $date_field ] ) ? $data[ $date_field ] : false;
		$email       = ! empty( $data[ $email_field ] ) ? $data[ $email_field ] : false;
		$provider_id = false;

		if ( ! $service_id || ! $date || ! $email || ! is_email( $email ) ) {
			$notifications->log[] = false;
			return false;
		}

		if ( $provider_field ) {

			if ( '_manual_input' === $provider_field ) {
				$provider_id = ! empty( $args['appointment_provider_id'] ) ? absint( $args['appointment_provider_id'] ) : 0;
			} else {
				$provider_id = ! empty( $data[ $provider_field ] ) ? $data[ $provider_field ] : false;
			}

			if ( ! $provider_id ) {
				$notifications->log[] = false;
				return false;
			}

		}

		$date = explode( '|', $date );
		$day  = $date[0];
		$slot = $date[1];

		$duration = get_post_meta( $service_id, '_service_duration', true );

		if ( ! $duration ) {
			$duration = Plugin::instance()->settings->get( 'default_slot' );
		}

		$duration = absint( $duration );
		$slot_end = $slot + $duration;

		$new_appointment = array(
			'service'    => $service_id,
			'provider'   => $provider_id,
			'user_email' => sanitize_email( $email ),
			'date'       => $day,
			'slot'       => $slot,
			'slot_end'   => $slot_end,
		);

		$db_columns = Plugin::instance()->settings->get( 'db_columns' );

		if ( ! empty( $db_columns ) ) {
			foreach ( $db_columns as $column ) {

				$arg        = 'appointment_custom_field_' . $column;
				$field_name = ! empty( $args[ $arg ] ) ? $args[ $arg ] : false;

				$new_appointment[ $column ] = ! empty( $data[ $field_name ] ) ? esc_attr( $data[ $field_name ] ) : '';
			}
		}

		if ( Plugin::instance()->wc->get_status() && Plugin::instance()->wc->get_product_id() ) {
			$new_appointment['status'] = 'on-hold';
		} elseif ( $notifications->handler->has_gateway() ) {

			$new_appointment['status']   = 'on-hold';

			if ( ! empty( $notifications->handler->form_data['inserted_post_id'] ) ) {
				$new_appointment['order_id'] = $notifications->handler->form_data['inserted_post_id'];
			}

		}

		if ( ! Plugin::instance()->db->appointment_available( $new_appointment ) ) {
			$notifications->log[] = $notifications->set_specific_status( 'Appointment time already taken' );
			return false;
		}

		$appointment_id = Plugin::instance()->db->add_appointment( $new_appointment );

		$notifications->handler->form_data['appointment_id'] = $appointment_id;
		$notifications->data['appointment_id']               = $appointment_id;

		do_action( 'jet-apb/form/notification/success', array(
			'ID'         => $appointment_id,
			'service'    => $service_id,
			'provider'   => $provider_id,
			'slot'       => $slot,
			'date'       => $day,
		), $notifications, $args );

		$notifications->log[] = true;

	}

	/**
	 * Register new notification type
	 *
	 * @return [type] [description]
	 */
	public function register_notification( $notifications ) {
		$notifications['insert_appointment'] = __( 'Insert appointment' );
		return $notifications;
	}

	/**
	 * Render additional edit fields
	 *
	 * @return [type] [description]
	 */
	public function edit_fields() {
		include JET_APB_PATH . 'templates/admin/form/edit-fields.php';
	}

	/**
	 * Render additional notification fields
	 *
	 * @return [type] [description]
	 */
	public function notification_fields() {

		$additional_db_columns = Plugin::instance()->settings->get( 'db_columns' );
		$wc_integration        = Plugin::instance()->settings->get( 'wc_integration' );

		include JET_APB_PATH . 'templates/admin/form/notification-fields.php';
	}

	/**
	 * Register new dates fields
	 *
	 * @return [type] [description]
	 */
	public function register_form_fields( $fields ) {

		$fields['appointment_date']     = __( 'Appointment date', 'jet-appointments-booking' );
		$fields['appointment_provider'] = __( 'Appointment provider', 'jet-appointments-booking' );

		return $fields;

	}

	/**
	 * Render provider field tempalte
	 *
	 * @return [type] [description]
	 */
	public function provider_field_template( $template, $args, $builder ) {

		if ( ! $this->provider_done ) {

			add_action( 'wp_footer', function() {

				ob_start();
				include JET_APB_PATH . 'assets/js/public/providers-init.js';
				$init_script = ob_get_clean();

				printf( '<script>%s</script>', $init_script );

			}, 99 );

			$this->provider_done = true;
		}

		$service_data = $this->get_service_field_data( $args );
		$placeholder  = ! empty( $args['placeholder'] ) ? $args['placeholder'] : __( 'Select...', 'jet-engine' );

		$dataset = array(
			'service'     => $service_data['form_field'],
			'api'         => Plugin::instance()->rest_api->get_urls(),
			'placeholder' => $placeholder,
		);

		$is_required = $builder->get_required_val( $args );
		$args        = 'name="' . $args['name'] . '"';

		if ( $is_required ) {
			$args .= ' required';
		}

		$providers_list = '';

		if ( ! empty( $service_data['id'] ) ) {
			$providers = Plugin::instance()->tools->get_providers_for_service( $service_data['id'] );

			if ( ! empty( $providers ) ) {
				$providers_list .= '<option value="">' . $placeholder . '</option>';
				foreach ( $providers as $provider ) {
					$providers_list .= '<option value="' . $provider->ID . '">' . $provider->post_title . '</option>';
				}
			}

		}

		return sprintf(
			'<select class="appointment-provider jet-form__field select-field" %2$s data-args="%1$s">%3$s</select>',
			htmlspecialchars( json_encode( $dataset ) ),
			$args,
			$providers_list
		);

	}

	public function calendar_assets() {

		ob_start();
		include JET_APB_PATH . 'assets/js/public/appointments-init.js';
		$init_script = ob_get_clean();

		$handle = 'vanilla-calendar';

		wp_enqueue_script(
			$handle,
			JET_APB_URL . 'assets/js/public/vanilla-calendar.js',
			array( 'jquery' ),
			JET_APB_VERSION,
			true
		);

		wp_add_inline_script( $handle, $init_script );

		wp_localize_script( $handle, 'JetAPBData', array(
			'api'                 => Plugin::instance()->rest_api->get_urls(),
			'css'                 => JET_APB_URL . 'assets/css/vanilla-calendar.css',
			'week_days'           => Plugin::instance()->calendar->get_week_days(),
			'start_of_week'       => get_option( 'start_of_week', 1 ),
			'available_week_days' => Plugin::instance()->calendar->get_available_week_days(),
		) );

	}

	/**
	 * Return lift of excluded services
	 * @return [type] [description]
	 */
	public function get_allowed_services() {

		$provider_cpt = Plugin::instance()->settings->get( 'providers_cpt' );

		if ( ! $provider_cpt ) {
			return false;
		}

		if ( ! is_singular( $provider_cpt ) ) {
			return false;
		}

		return Plugin::instance()->tools->get_services_for_provider( get_the_ID() );

	}

	/**
	 * Render field template
	 *
	 * @return [type] [description]
	 */
	public function date_field_template( $template, $args, $builder ) {

		if ( ! $this->date_done ) {
			$this->calendar_assets();
			$this->date_done = true;
		}

		$service_data  = $this->get_service_field_data( $args );
		$provider_data = $this->get_provider_field_data( $args );
		$excl_dates    = Plugin::instance()->calendar->get_excluded_dates( $service_data['id'], $provider_data['id'] );

		$dataset = array(
			'excludedDates'   => $excl_dates,
			'service'         => $service_data['form_field'],
			'providerIsset'   => $provider_data['is_set'],
			'provider'        => $provider_data['form_field'],
			'inputName'       => $args['name'],
			'allowedServices' => $this->get_allowed_services(),
		);

		printf(
			'<div class="appointment-calendar jet-apb-calendar" data-args="%1$s"></div>',
			htmlspecialchars( json_encode( $dataset ) )
		);

	}

	/**
	 * Returns service field data
	 *
	 * @param  array  $args [description]
	 * @return [type]       [description]
	 */
	public function get_service_field_data( $args = array() ) {

		$service_id    = false;
		$service       = false;
		$service_field = ! empty( $args['appointment_service_field'] ) ? $args['appointment_service_field'] : 'current_post_id';

		if ( 'current_post_id' === $service_field ) {
			$service_id = get_the_ID();
			$service    = array(
				'id' => $service_id,
			);
		} elseif ( 'manual_input' === $service_field ) {
			$service_id = ! empty( $args['appointment_service_id'] ) ? absint( $args['appointment_service_id'] ) : false;

			if ( $service_id ) {
				$service = array(
					'id' => $service_id,
				);
			}
		} else {

			$field = ! empty( $args['appointment_form_field'] ) ? $args['appointment_form_field'] : false;

			if ( $field ) {
				$service = array(
					'field' => $field,
				);
			} else {
				$service = false;
			}

		}

		return array(
			'id'         => $service_id,
			'form_field' => $service,
		);

	}

	/**
	 * Return parseed provider data from arguments
	 *
	 * @param  array  $args [description]
	 * @return [type]       [description]
	 */
	public function get_provider_field_data( $args = array() ) {

		$provider_cpt   = Plugin::instance()->settings->get( 'providers_cpt' );
		$provider_id    = false;
		$provider       = false;
		$provider_field = ! empty( $args['appointment_provider_field'] ) ? $args['appointment_provider_field'] : '';

		if ( ! $provider_cpt ) {
			return array(
				'is_set'     => false,
				'id'         => false,
				'form_field' => false,
			);
		}

		if ( 'current_post_id' === $provider_field ) {
			$provider_id = get_the_ID();
			$provider    = array(
				'id' => $provider_id,
			);
		} elseif ( 'manual_input' === $provider_field ) {
			$provider_id = ! empty( $args['appointment_provider_id'] ) ? absint( $args['appointment_provider_id'] ) : false;
			if ( $provider_id ) {
				$provider = array(
					'id' => $provider_id,
				);
			}
		} else {

			$field = ! empty( $args['appointment_provider_form_field'] ) ? $args['appointment_provider_form_field'] : false;

			if ( $field ) {
				$provider = array(
					'field' => $field,
				);
			} else {
				$provider = false;
			}

		}

		if ( $provider || $provider_id ) {
			$is_set = true;
		} else {
			$is_set = false;
		}

		return array(
			'is_set'     => $is_set,
			'id'         => $provider_id,
			'form_field' => $provider,
		);

	}

}
