<?php
namespace JET_APB;

/**
 * Tools class
 */
class Tools {

	/**
	 * Get services for provider ID
	 *
	 * @param  integer $provider_id [description]
	 * @return [type]              [description]
	 */
	public function get_services_for_provider( $provider_id = 0 ) {

		$services_cpt  = Plugin::instance()->settings->get( 'services_cpt' );
		$providers_cpt = Plugin::instance()->settings->get( 'providers_cpt' );

		if ( ! $provider_id || ! $services_cpt || ! $providers_cpt ) {
			return false;
		}

		$services = jet_engine()->relations->get_related_posts( array(
			'post_type_1' => $services_cpt,
			'post_type_2' => $providers_cpt,
			'post_id'     => $provider_id,
			'from'        => $services_cpt,
		) );

		return $services;

	}

	/**
	 * Get providers list for passed services ID
	 *
	 * @param  integer $service_id [description]
	 * @return [type]              [description]
	 */
	public function get_providers_for_service( $service_id = 0 ) {

		$services_cpt  = Plugin::instance()->settings->get( 'services_cpt' );
		$providers_cpt = Plugin::instance()->settings->get( 'providers_cpt' );

		if ( ! $service_id || ! $services_cpt || ! $providers_cpt ) {
			return array();
		}

		$providers = jet_engine()->relations->get_related_posts( array(
			'post_type_1' => $services_cpt,
			'post_type_2' => $providers_cpt,
			'post_id'     => $service_id,
			'from'        => $providers_cpt,
		) );

		$result = array();

		if ( ! empty( $providers ) ) {

			global $wpdb;

			$query  = implode( ', ', $providers );
			$table  = $wpdb->posts;
			$result = $wpdb->get_results( "SELECT ID, post_title FROM $table WHERE `ID` IN ( $query );" );

		}

		if ( ! is_array( $result ) ) {
			return array();
		} else {
			return $result;
		}

	}

	/**
	 * Returns services label
	 * @return [type] [description]
	 */
	public function get_services_label() {

		$services_cpt = Plugin::instance()->settings->get( 'services_cpt' );

		if ( ! $services_cpt ) {
			return null;
		}

		$services = get_post_type_object( $services_cpt );

		if ( ! $services ) {
			return null;
		}

		return $services->labels->singular_name;

	}

	/**
	 * Returns providers label
	 * @return [type] [description]
	 */
	public function get_providers_label() {

		$providers_cpt = Plugin::instance()->settings->get( 'providers_cpt' );

		if ( ! $providers_cpt ) {
			return null;
		}

		$providers = get_post_type_object( $providers_cpt );

		if ( ! $providers ) {
			return null;
		}

		return $providers->labels->singular_name;

	}

	/**
	 * Returns verbosed slot by timestatmp
	 * @return [type] [description]
	 */
	public function get_verbosed_slot( $slot ) {
		$format = get_option( 'date_format', 'F j, Y' ) . ' ' . get_option( 'time_format', 'H:i' );
		return date_i18n( $format, $slot );
	}

	public function get_posts( $from = 'services' ) {

		$post_type = false;

		switch ( $from ) {
			case 'services':
				$post_type = Plugin::instance()->settings->get( 'services_cpt' );
				break;

			case 'providers':
				$post_type = Plugin::instance()->settings->get( 'providers_cpt' );
				break;
		}

		if ( ! $post_type ) {
			return array();
		}

		$posts = get_posts( array(
			'post_type'      => $post_type,
			'posts_per_page' => -1,
		) );

		if ( empty( $posts ) ) {
			return array();
		} else {
			return wp_list_pluck( $posts, 'post_title', 'ID' );
		}

	}

	/**
	 * Returns path to template file.
	 *
	 * @return string|bool
	 */
	public function get_template( $name = null ) {

		$template_path = apply_filters( 'jet-apb/tools/template-path', 'jet-appointments-booking' );
		$template      = locate_template( $template_path . $name );

		if ( ! $template ) {
			$template = JET_APB_PATH . 'templates/public/' . $name;
		}

		if ( file_exists( $template ) ) {
			return $template;
		} else {
			return false;
		}
	}

}
