<?php
namespace JET_APB;

/**
 * WooCommerce integration class
 */
class WC_Integration {

	private $is_enbaled             = false;
	private $product_id             = 0;
	private $price_adjusted         = false;
	private $appointmet_product_key = '_is_jet_appointment';
	private $data_key               = 'appointment_data';
	private $price_key              = 'wc_appointment_price';
	private $form_data_key          = 'appointment_form_data';

	/**
	 * Constructir for the class
	 */
	public function __construct() {

		if ( ! class_exists( '\WooCommerce' ) ) {
			return;
		}

		add_action( 'jet-apb/settings/before-write', array( $this, 'maybe_create_appointment_product' ) );

		$this->set_status();

		if ( ! $this->get_status() || ! $this->get_product_id() ) {
			return;
		}

		// Form-related
		add_action( 'jet-apb/form/notification/success', array( $this, 'process_wc_notifictaion' ), 10, 3 );

		// Cart-related
		add_filter( 'woocommerce_get_item_data', array( $this, 'add_formatted_cart_data' ), 10, 2 );
		add_filter( 'woocommerce_get_cart_contents', array( $this, 'set_appointment_price' ) );

		// Order-related
		add_action( 'woocommerce_checkout_order_processed', array( $this, 'process_order' ), 10, 3 );
		add_action( 'woocommerce_thankyou', array( $this, 'order_details' ), 0 );
		add_action( 'woocommerce_view_order', array( $this, 'order_details' ), 0 );
		add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'admin_order_details' ) );
		add_action( 'woocommerce_order_status_changed', array( $this, 'update_status_on_order_update' ), 10, 4 );

	}

	/**
	 * Update an appointment status on related order update
	 *
	 * @return [type] [description]
	 */
	public function update_status_on_order_update( $order_id, $old_status, $new_status, $order ) {

		$appointment = $this->get_appointment_by_order_id( $order_id );

		if ( ! $appointment ) {
			return;
		}

		$this->set_order_data( $appointment, $order_id, $order );

		if ( in_array( $new_status, Plugin::instance()->statuses->invalid_statuses() ) ) {
			Plugin::instance()->db->remove_appointment_date_from_excluded( $appointment );
		}

		if ( in_array( $old_status, Plugin::instance()->statuses->invalid_statuses() ) && in_array( $new_status, Plugin::instance()->statuses->valid_statuses() ) ) {
			Plugin::instance()->db->maybe_exclude_appointment_date( $appointment );
		}

	}

	/**
	 * Process new order creation
	 *
	 * @param  [type] $order [description]
	 * @param  [type] $data  [description]
	 * @return [type]        [description]
	 */
	public function process_order( $order_id, $data, $order ) {

		$cart = WC()->cart->get_cart_contents();

		foreach ( $cart as $item ) {
			if ( ! empty( $item[ $this->data_key ] ) ) {
				$this->set_order_data(
					$item[ $this->data_key ],
					$order_id,
					$order
				);
			}
		}

	}

	/**
	 * Setup order data
	 */
	public function set_order_data( $app_data, $order_id, $order ) {

		$service  = ! empty( $app_data['service'] ) ? absint( $app_data['service'] ) : false;
		$provider = ! empty( $app_data['provider'] ) ? absint( $app_data['provider'] ) : false;
		$date     = ! empty( $app_data['date'] ) ? absint( $app_data['date'] ) : false;
		$slot     = ! empty( $app_data['slot'] ) ? absint( $app_data['slot'] ) : false;
		$id       = ! empty( $app_data['ID'] ) ? absint( $app_data['ID'] ) : false;

		if ( ! $service || ! $date || ! $slot ) {
			return;
		}

		$where = array(
			'service' => $service,
			'date'    => $date,
			'slot'    => $slot,
		);

		if ( $id ) {
			$where['ID'] = $id;
		}

		if ( $provider ) {
			$where['provider'] = $provider;
		}

		Plugin::instance()->db->appointments->update( array(
			'order_id' => $order_id,
			'status'   => $order->get_status(),
		), $where );

	}

	/**
	 * Set custom price per appointemnt
	 *
	 * @param [type] $cart [description]
	 */
	public function set_appointment_price( $cart_items ) {

		if ( $this->price_adjusted ) {
			return $cart_items;
		}

		if ( ! empty( $cart_items ) ) {
			foreach ( $cart_items as $item ) {
				if ( ! empty( $item[ $this->data_key ] ) ) {

					if ( ! empty( $item[ $this->price_key ] ) ) {
						$price = $item[ $this->price_key ];
					} else {

						$data    = $item[ $this->data_key ];
						$service = ! empty( $data['service'] ) ? $data['service'] : 0;
						$price   = get_post_meta( $service, '_app_price', true );

					}

					if ( $price ) {
						$item['data']->set_price( floatval( $price ) );
					}

					$this->price_adjusted = true;
				}
			}
		}

		return $cart_items;

	}

	/**
	 * Add appointment infor,ation into cart meta data
	 *
	 * @param [type] $item_data [description]
	 * @param [type] $cart_item [description]
	 */
	public function add_formatted_cart_data( $item_data, $cart_item ) {

		if ( ! empty( $cart_item[ $this->data_key ] ) ) {
			$item_data = array_merge(
				$item_data,
				$this->get_formatted_appointment_info(
					$cart_item[ $this->data_key ],
					$cart_item[ $this->form_data_key ]
				)
			);
		}

		return $item_data;

	}

	/**
	 * Show appointment-related order details on order page
	 *
	 * @param  [type] $order_id [description]
	 * @return [type]           [description]
	 */
	public function order_details( $order_id ) {

		$appointment = $this->get_appointment_by_order_id( $order_id );

		if ( ! $appointment ) {
			return;
		}

		$service        = get_the_title( $appointment['service'] );
		$provider       = ! empty( $appointment['provider'] ) ? get_the_title( $appointment['provider'] ) : false;
		$slot           = Plugin::instance()->tools->get_verbosed_slot( absint( $appointment['slot'] ) );
		$service_label  = Plugin::instance()->tools->get_services_label();
		$provider_label = Plugin::instance()->tools->get_providers_label();

		include Plugin::instance()->tools->get_template( 'order-details.php' );

	}

	/**
	 * Returns appointment detail by order id
	 *
	 * @return [type] [description]
	 */
	public function get_appointment_by_order_id( $order_id ) {

		$appointment = Plugin::instance()->db->get_appointment_by( 'order_id', $order_id );

		if ( ! $appointment['service'] ) {
			return false;
		}

		return $appointment;

	}

	/**
	 * Admin order details
	 *
	 * @param  [type] $order [description]
	 * @return [type]        [description]
	 */
	public function admin_order_details( $order ) {

		$order_id    = $order->get_id();
		$appointment = $this->get_appointment_by_order_id( $order_id );

		if ( ! $appointment ) {
			return;
		}

		$service        = get_the_title( $appointment['service'] );
		$provider       = ! empty( $appointment['provider'] ) ? get_the_title( $appointment['provider'] ) : false;
		$slot           = Plugin::instance()->tools->get_verbosed_slot( absint( $appointment['slot'] ) );
		$service_label  = Plugin::instance()->tools->get_services_label();
		$provider_label = Plugin::instance()->tools->get_providers_label();

		include JET_APB_PATH . 'templates/admin/order/details.php';

	}

	/**
	 * Get formatted appointment information
	 *
	 * @return [type] [description]
	 */
	public function get_formatted_appointment_info( $data = array(), $form_data = array() ) {

		$service  = ! empty( $data['service'] ) ? absint( $data['service'] ) : false;
		$provider = ! empty( $data['provider'] ) ? absint( $data['provider'] ) : false;
		$date     = ! empty( $data['date'] ) ? absint( $data['date'] ) : false;
		$slot     = ! empty( $data['slot'] ) ? absint( $data['slot'] ) : false;
		$result   = array();

		if ( ! $service || ! $date || ! $slot ) {
			return;
		}

		$result[] = array(
			'key'     => __( 'Time', 'jet-engine' ),
			'display' => Plugin::instance()->tools->get_verbosed_slot( $slot ),
		);

		$result[] = array(
			'key'     => Plugin::instance()->tools->get_services_label(),
			'display' => get_the_title( $service ),
		);

		if ( $provider ) {
			$result[]      = array(
				'key'     => Plugin::instance()->tools->get_providers_label(),
				'display' => get_the_title( $provider ),
			);
		}

		return apply_filters( 'jet-booking/wc-integration/cart-info', $result, $data, $form_data );

	}

	/**
	 * Process WC-related notification part
	 *
	 * @param  [type] $data [description]
	 * @return [type]       [description]
	 */
	public function process_wc_notifictaion( $data, $notifications, $n_args ) {

		if ( ! $this->get_status() || ! $this->get_product_id() ) {
			return;
		}

		$cart_item_data = array(
			$this->data_key      => $data,
			$this->form_data_key => $notifications->data,
		);

		$price_field = ! empty( $n_args['appointment_wc_price'] ) ? $n_args['appointment_wc_price'] : false;
		$price       = false;

		if ( $price_field && isset( $notifications->data[ $price_field ] ) ) {
			$price = floatval( $notifications->data[ $price_field ] );
		}

		if ( $price ) {
			$cart_item_data[ $this->price_key ] = $price;
		}

		WC()->cart->empty_cart();
		WC()->cart->add_to_cart( $this->get_product_id(), 1, 0, array(), $cart_item_data );

		add_filter( 'jet-engine/forms/handler/query-args', function( $query_args, $args, $handler ) {

			if ( 'success' !== $args['status'] ) {
				return $query_args;
			}

			$url = wc_get_checkout_url();

			if ( $handler->is_ajax ) {
				$query_args['redirect'] = $url;
				return $query_args;
			} else {
				wp_redirect( $url );
				die();
			}

		}, 10, 3 );

	}

	/**
	 * Check if we need to create new Appointment product
	 *
	 * @return [type] [description]
	 */
	public function maybe_create_appointment_product( $settings ) {

		$new_status = $settings->get( 'wc_integration' );

		if ( ! $new_status ) {
			return;
		}

		$wc_product_id = $settings->get( 'wc_product_id' );
		$wc_product_id = false;

		if ( $wc_product_id ) {
			return;
		}

		$product_id = $this->get_product_id_from_db();

		if ( ! $product_id ) {
			$product_id = $this->create_appointment_product();
		}

		$settings->update( 'wc_product_id', $product_id, false );

	}

	/**
	 * Try to get previousle created product ID in db.
	 * @return [type] [description]
	 */
	public function get_product_id_from_db() {

		global $wpdb;

		$table      = $wpdb->postmeta;
		$key        = $this->appointmet_product_key;
		$product_id = $wpdb->get_var( "SELECT `post_id` FROM $table WHERE `meta_key` = '$key';" );

		if ( ! $product_id ) {
			return false;
		}

		if ( 'product' !== get_post_type( $product_id ) ) {
			return false;
		}

		return absint( $product_id );
	}

	/**
	 * Returns product name
	 *
	 * @return [type] [description]
	 */
	public function get_product_name() {

		return apply_filters(
			'jet-apb/wc-integration/product-name',
			__( 'Appointment', 'jet-engine' )
		);

	}

	/**
	 * Create new appointment product
	 *
	 * @return [type] [description]
	 */
	public function create_appointment_product() {

		$product = new \WC_Product_Simple( 0 );

		$product->set_name( $this->get_product_name() );
		$product->set_status( 'publish' );
		$product->set_price( 1 );
		$product->set_regular_price( 1 );
		$product->set_slug( sanitize_title( $this->get_product_name() ) );

		$product->save();

		$product_id = $product->get_id();

		if ( $product_id ) {
			update_post_meta( $product_id, $this->appointmet_product_key, true );
		}

		return $product_id;

	}

	/**
	 * Set WC integration status
	 */
	public function set_status() {

		$is_enbaled       = Plugin::instance()->settings->get( 'wc_integration' );
		$product_id       = Plugin::instance()->settings->get( 'wc_product_id' );
		$this->is_enbaled = filter_var( $is_enbaled, FILTER_VALIDATE_BOOLEAN );
		$this->product_id = $product_id;

	}

	/**
	 * Return WC integration status
	 *
	 * @return [type] [description]
	 */
	public function get_status() {
		return $this->is_enbaled;
	}

	/**
	 * Return WC integration product
	 *
	 * @return [type] [description]
	 */
	public function get_product_id() {
		return $this->product_id;
	}

}
